package com.perforce.ts.jiraAccess;

import com.atlassian.jira.rest.client.api.domain.BasicComponent;
import com.atlassian.jira.rest.client.api.domain.Comment;
import com.atlassian.jira.rest.client.api.domain.Issue;
import com.atlassian.jira.rest.client.api.domain.IssueField;
import com.atlassian.jira.rest.client.api.domain.Version;
import com.perforce.ts.common.Constants;
import com.perforce.ts.common.Utils;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.codehaus.jettison.json.JSONException;
import org.codehaus.jettison.json.JSONObject;

/**
 * Structure of a Returned Issue
 * Usage: if defect not found, instantiate with fields=null
 * qne when checking use defect.exists()
 *
 * @author jbrown
 */
public class DiffBugsRestDefectResult {

    public DiffBugsRestDefectResult(String defectId, String message, Issue issue) {
        this.defectId = defectId;
        this.message = message;
        this.fields = addPropFields(issue);
    }

    public boolean exists() {
        return (this.fields != null && this.fields.size() > 0);
    }
    public String defectId;
    public String message;

    /*
    protected IssueField[] fields;

    public IssueField[] getFields() {
        return fields;
    }
     */
    protected Map<String, String[]> fields;

    public Map<String, String[]> getFields() {
        return fields;
    }

    public static synchronized  Date parseFieldDate(String date) throws ParseException {
        Date d = null;
        if (date != null) {
            d = DATE_FORMATTER_SHORT.parse(date);
        }
        return d;
    }

    /**
     * Date pattern.
     */
    // Rest date format sample:  2016-08-29T16:56:54.464-0700
    public static final String DATE_PATTERN = "yyyy-MM-dd'T'HH:mm:ss"; //.SSS and TZ omitted 
    public static final String DATE_PATTERN_SHORT = "yyyy/MM/dd HH:mm:ss";

    private static final DateFormat DATE_FORMATTER = new SimpleDateFormat(DATE_PATTERN);
    private static final DateFormat DATE_FORMATTER_SHORT = new SimpleDateFormat(DATE_PATTERN_SHORT);

    /**
     * Date pattern for JIRA Query Language (JQL).
     */
    public static final String JQL_DATE_PATTERN = "yyyy/MM/dd HH:mm";

    private Map<String, String[]> addPropFields(Issue issue) {

        Map<String, String[]> fieldValueMap = new HashMap();

        for (IssueField field : issue.getFields()) {
            Object valObj = field.getValue();
            String val = null;
            if (valObj != null) {
                if  ( valObj instanceof JSONObject) {
                    JSONObject jobj = (JSONObject)valObj;
                    try {
                        val = jobj.getString("value");
                    } catch (JSONException ex) {
                        val = null;
                    }
                    
                } else {
                    val = valObj.toString();
                } 
                if ( val != null) {
                fieldValueMap.put(field.getName(), new String[]{val});
                }
            }
        }
        if (issue.getKey() != null) {
            fieldValueMap.put(Constants.ISSUE_FIELD_KEY, new String[]{issue.getKey()});
        }
        if (issue.getReporter() != null) {
            fieldValueMap.put(Constants.ISSUE_FIELD_REPORTER, new String[]{issue.getReporter().getName()});
        }
        if (issue.getAssignee() != null) {
            fieldValueMap.put(Constants.ISSUE_FIELD_ASSIGNEE, new String[]{issue.getAssignee().getName()});
        }
        if (issue.getSummary() != null) {
            fieldValueMap.put(Constants.ISSUE_FIELD_SUMMARY, new String[]{issue.getSummary()});
        }
        if (issue.getDescription() != null) {
            fieldValueMap.put(Constants.ISSUE_FIELD_DESCRIPTION, new String[]{issue.getDescription()});
        }
        IssueField envField = issue.getFieldByName(Constants.ISSUE_FIELD_ENVIRONMENT);
        if (envField != null && envField.getValue() != null) {
            fieldValueMap.put(Constants.ISSUE_FIELD_ENVIRONMENT, new String[]{envField.getValue().toString()});
        }
        Iterable<Comment> comments = issue.getComments();
        if (comments != null) {
            StringBuilder sb = new StringBuilder();
            int count = 0;
            for (Comment comment : comments) {
                if (count > 0) {
                    sb.append(Constants.COMMENT_SEPARATOR);
                }
                sb.append(comment.getBody());
                count++;
            }
            fieldValueMap.put(Constants.ISSUE_FIELD_COMMENTS, new String[]{sb.toString()});
        }
        Iterable<Version> affectsVersions = issue.getAffectedVersions();
        if (affectsVersions != null) {
            StringBuilder sb = new StringBuilder();
            int count = 0;
            for (Version version : affectsVersions) {
                if (count > 0) {
                    sb.append(Constants.MULTI_SELECT_SEPARATOR);
                }
                sb.append(version.getName());
                count++;
            }
            fieldValueMap.put(Constants.ISSUE_FIELD_AFFECTSVERSIONS, new String[]{sb.toString()});
        }
        Iterable<Version> fixVersions = issue.getFixVersions();
        if (fixVersions != null) {
            StringBuilder sb = new StringBuilder();
            int count = 0;
            for (Version version : fixVersions) {
                if (count > 0) {
                    sb.append(Constants.MULTI_SELECT_SEPARATOR);
                }
                sb.append(version.getName());
                count++;
            }
            fieldValueMap.put(Constants.ISSUE_FIELD_FIXVERSIONS, new String[]{sb.toString()});
        }
        Iterable<BasicComponent> components = issue.getComponents();
        if (components != null) {
            StringBuilder sb = new StringBuilder();
            int count = 0;
            for (BasicComponent component : components) {
                if (count > 0) {
                    sb.append(Constants.MULTI_SELECT_SEPARATOR);
                }
                sb.append(component.getName());
                count++;
            }
            fieldValueMap.put(Constants.ISSUE_FIELD_COMPONENTS, new String[]{sb.toString()});
        }
        if (issue.getDueDate() != null) {
            fieldValueMap.put(Constants.ISSUE_FIELD_DUEDATE, new String[]{
                Utils.formatDate(issue.getDueDate().toDate(), Constants.DATE_PATTERN)});
        }
        if (issue.getUpdateDate() != null) {
            fieldValueMap.put(Constants.ISSUE_FIELD_UPDATED, new String[]{
                Utils.formatDate(issue.getUpdateDate().toDate(), Constants.DATE_PATTERN)});
        }
        if (issue.getIssueType() != null) {
            fieldValueMap.put(Constants.ISSUE_FIELD_ISSUETYPE, new String[]{issue.getIssueType().getName()});
        }
        if (issue.getPriority() != null) {
            fieldValueMap.put(Constants.ISSUE_FIELD_PRIORITY, new String[]{issue.getPriority().getName()});
        }
        if (issue.getStatus() != null) {
            fieldValueMap.put(Constants.ISSUE_FIELD_STATUS, new String[]{issue.getStatus().getName()});
        }
        if (issue.getResolution() != null) {
            fieldValueMap.put(Constants.ISSUE_FIELD_RESOLUTION, new String[]{issue.getResolution().getName()});
        }
        return fieldValueMap;
    }

}
